@@ -210,6 +210,7 @@ else:
 
 
 def get_procfs_path():
+    """Return updated psutil.PROCFS_PATH constant."""
     return sys.modules['psutil'].PROCFS_PATH
 
 
@@ -234,6 +235,9 @@ def readlink(path):
 
 
 def file_flags_to_mode(flags):
+    """Convert file's open() flags into a readable string.
+    Used by Process.open_files().
+    """
     modes_map = {os.O_RDONLY: 'r', os.O_WRONLY: 'w', os.O_RDWR: 'w+'}
     mode = modes_map[flags & (os.O_RDONLY | os.O_WRONLY | os.O_RDWR)]
     if flags & os.O_APPEND:
@@ -243,26 +247,26 @@ def file_flags_to_mode(flags):
     return mode
 
 
-def get_sector_size():
+def get_sector_size(partition):
+    """Return the sector size of a partition.
+    Used by disk_io_counters().
+    """
     try:
-        with open(b"/sys/block/sda/queue/hw_sector_size") as f:
+        with open("/sys/block/%s/queue/hw_sector_size" % partition, "rt") as f:
             return int(f.read())
     except (IOError, ValueError):
         # man iostat states that sectors are equivalent with blocks and
-        # have a size of 512 bytes since 2.4 kernels. This value is
-        # needed to calculate the amount of disk I/O in bytes.
+        # have a size of 512 bytes since 2.4 kernels.
         return 512
 
 
-SECTOR_SIZE = get_sector_size()
-
-
 @memoize
 def set_scputimes_ntuple(procfs_path):
-    """Return a namedtuple of variable fields depending on the
-    CPU times available on this Linux kernel version which may be:
+    """Set a namedtuple of variable fields depending on the CPU times
+    available on this Linux kernel version which may be:
     (user, nice, system, idle, iowait, irq, softirq, [steal, [guest,
      [guest_nice]]])
+    Used by cpu_times() function.
     """
     global scputimes
     with open_binary('%s/stat' % procfs_path) as f:
@@ -279,24 +283,25 @@ def set_scputimes_ntuple(procfs_path):
         # Linux >= 3.2.0
         fields.append('guest_nice')
     scputimes = namedtuple('scputimes', fields)
-    return scputimes
 
 
 def cat(fname, fallback=_DEFAULT, binary=True):
-    """Return file content."""
+    """Return file content.
+    fallback: the value returned in case the file does not exist or
+              cannot be read
+    binary: whether to open the file in binary or text mode.
+    """
     try:
-        f = open_binary(fname) if binary else open_text(fname)
+        with open_binary(fname) if binary else open_text(fname) as f:
+            return f.read().strip()
     except IOError:
         if fallback != _DEFAULT:
             return fallback
         raise
-    else:
-        with f:
-            return f.read().strip()
 
 
 try:
-    scputimes = set_scputimes_ntuple("/proc")
+    set_scputimes_ntuple("/proc")
 except Exception:
     # Don't want to crash at import time.
     traceback.print_exc()
@@ -474,6 +479,7 @@ def virtual_memory():
 
 
 def swap_memory():
+    """Return swap memory metrics."""
     _, _, _, _, total, free, unit_multiplier = cext.linux_sysinfo()
     total *= unit_multiplier
     free *= unit_multiplier
@@ -607,6 +613,7 @@ def cpu_count_physical():
 
 
 def cpu_stats():
+    """Return various CPU stats as a named tuple."""
     with open_binary('%s/stat' % get_procfs_path()) as f:
         ctx_switches = None
         interrupts = None
@@ -629,6 +636,10 @@ def cpu_stats():
 if os.path.exists("/sys/devices/system/cpu/cpufreq"):
 
     def cpu_freq():
+        """Return frequency metrics for all CPUs.
+        Contrarily to other OSes, Linux updates these values in
+        real-time.
+        """
         # scaling_* files seem preferable to cpuinfo_*, see:
         # http://unix.stackexchange.com/a/87537/168884
         ret = []
@@ -1027,15 +1038,16 @@ def disk_io_counters():
             raise ValueError("not sure how to interpret line %r" % line)
 
         if name in partitions:
-            rbytes = rbytes * SECTOR_SIZE
-            wbytes = wbytes * SECTOR_SIZE
+            sector_size = get_sector_size(name)
+            rbytes = rbytes * sector_size
+            wbytes = wbytes * sector_size
             retdict[name] = (reads, writes, rbytes, wbytes, rtime, wtime,
                              reads_merged, writes_merged, busy_time)
     return retdict
 
 
 def disk_partitions(all=False):
-    """Return mounted disk partitions as a list of namedtuples"""
+    """Return mounted disk partitions as a list of namedtuples."""
     fstypes = set()
     with open_text("%s/filesystems" % get_procfs_path()) as f:
         for line in f:
@@ -1081,13 +1093,17 @@ def sensors_temperatures():
       difficult to parse
     """
     ret = collections.defaultdict(list)
-    # Will return an empty dict if path does not exist.
-    basenames = sorted(set(
-        [x.split('_')[0] for x in
-         glob.glob('/sys/class/hwmon/hwmon*/temp*_*')]))
+    basenames = glob.glob('/sys/class/hwmon/hwmon*/temp*_*')
+    if not basenames:
+        # CentOS has an intermediate /device directory:
+        # https://github.com/giampaolo/psutil/issues/971
+        basenames = glob.glob('/sys/class/hwmon/hwmon*/device/temp*_*')
+
+    basenames = sorted(set([x.split('_')[0] for x in basenames]))
     for base in basenames:
-        unit_name = cat(os.path.join(os.path.dirname(base), 'name'))
-        label = cat(base + '_label', fallback='')
+        unit_name = cat(os.path.join(os.path.dirname(base), 'name'),
+                        binary=False)
+        label = cat(base + '_label', fallback='', binary=False)
         current = float(cat(base + '_input')) / 1000.0
         high = cat(base + '_max', fallback=None)
         critical = cat(base + '_crit', fallback=None)
@@ -1103,16 +1119,73 @@ def sensors_temperatures():
 
 
 def sensors_battery():
+    """Return battery information.
+    Implementation note: it appears /sys/class/power_supply/BAT0/
+    directory structure may vary and provide files with the same
+    meaning but under different names, see:
+    https://github.com/giampaolo/psutil/issues/966
+    """
+    null = object()
+
+    def multi_cat(*paths):
+        """Attempt to read the content of multiple files which may
+        not exist. If none of them exist return None.
+        """
+        for path in paths:
+            ret = cat(path, fallback=null)
+            if ret != null:
+                return int(ret) if ret.isdigit() else ret
+        return None
+
     root = os.path.join(POWER_SUPPLY_PATH, "BAT0")
     if not os.path.exists(root):
         return None
 
-    power_plugged = cat(os.path.join(POWER_SUPPLY_PATH, "AC0/online"),
-                        fallback=b"0") == b"1"
-    energy_now = int(cat(root + "/energy_now"))
-    power_now = int(cat(root + "/power_now"))
-    percent = int(cat(root + "/capacity"))
+    # Base metrics.
+    energy_now = multi_cat(
+        root + "/energy_now",
+        root + "/charge_now")
+    power_now = multi_cat(
+        root + "/power_now",
+        root + "/current_now")
+    energy_full = multi_cat(
+        root + "/energy_full",
+        root + "/charge_full")
+    if energy_now is None or power_now is None:
+        return None
 
+    # Percent. If we have energy_full the percentage will be more
+    # accurate compared to reading /capacity file (float vs. int).
+    if energy_full is not None:
+        try:
+            percent = 100.0 * energy_now / energy_full
+        except ZeroDivisionError:
+            percent = 0.0
+    else:
+        percent = int(cat(root + "/capacity", fallback=-1))
+        if percent == -1:
+            return None
+
+    # Is AC power cable plugged in?
+    # Note: AC0 is not always available and sometimes (e.g. CentOS7)
+    # it's called "AC".
+    power_plugged = None
+    online = multi_cat(
+        os.path.join(POWER_SUPPLY_PATH, "AC0/online"),
+        os.path.join(POWER_SUPPLY_PATH, "AC/online"))
+    if online is not None:
+        power_plugged = online == 1
+    else:
+        status = cat(root + "/status", fallback="", binary=False).lower()
+        if status == "discharging":
+            power_plugged = False
+        elif status in ("charging", "full"):
+            power_plugged = True
+
+    # Seconds left.
+    # Note to self: we may also calculate the charging ETA as per:
+    # https://github.com/thialfihar/dotfiles/blob/
+    #     013937745fd9050c30146290e8f963d65c0179e6/bin/battery.py#L55
     if power_plugged:
         secsleft = _common.POWER_TIME_UNLIMITED
     else:
